/**********************************************************************************/
/*    Test program for:															  */
/*	  Board: AVR-PX128A1														  */
/*    Manufacture: OLIMEX                                                   	  */
/*	  COPYRIGHT (C) 2008														  */
/*    Designed by: Engineer Penko T. Bozhkov                                      */
/*    Module Name    :  MultiMediaCard in SPI mode                                */
/*    File   Name    :  mmc.c                                                     */
/*    Revision       :  Rev.B                                                     */
/*    Date           :  20.03.2009					                              */
/*    Approximately Program Memory Usage(ROM): 4974 bytes. (Include all mmc test!)*/
/*	  Approximately Data Memory Usage(RAM): 546 bytes.	(Include all mmc test!)	  */
/**********************************************************************************/
//#include <avr/io.h>
//#include "iox128a1.h"
#include "io430.h"
#include "mmc.h"

volatile unsigned char mmc_write_buffer[512];
//volatile unsigned char mmc_read_buffer[512];
volatile unsigned char mmcReg_buffer[16];


/**********************************************************************************/
/*  Function name: mmcSendByte				                                      */
/*  	Parameters                                                                */
/*          Input   :  spiDataByte		                                          */
/*          Output  :  spiDataByte						                          */
/*	Action: Send to and receive from SD/MMC 8-bits data via SPI.				  */
/*	Author:	Penko Bozhkov														  */
/**********************************************************************************/
unsigned char mmcSendByte(unsigned char spiDataByte)
{
	//MMC_NSS_Low;
  	/* Start transmission */
	UCB0TXBUF = spiDataByte;
	/* Wait for transmission complete */
	while(UCB0STAT & 0x01);	// Wait SPI Interrupt Flag to be set, i.e. transmission complete
	spiDataByte = UCB0RXBUF;		// Read received byte
	// Return the byte read from the SPI bus
	//MMC_NSS_High;
	return spiDataByte;
}


/**********************************************************************************/
/*  Function name: mmcSendCmd				                                      */
/*  	Parameters                                                                */
/*          Input   :  cmd, data, crc	                                          */
/*          Output  :  No						                          		  */
/*	Action: Send command to SD/MMC via SPI.						  				  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. int i -> unsigned char i, 8-bits length instead of 16-bits */
/*					2. "for" cycle is modified in the opposite counting			  */
/**********************************************************************************/
void mmcSendCmd(const char cmd, unsigned long data, const char crc)
{
  char frame[6];
  char temp;
  unsigned char i;

  frame[0]=(cmd|0x40);				// Set bit<7> in command MSByte to one because of command description (pg. 69)
  for(i=1;i<=4;i++){				// Arrange "data" in massive "frame". "data" MSByte is placed in "frame[1]", LSByte in "frame[4]"
    temp=(char)(data>>(8*(4-i)));	// Type conversion from unsigned long to char.
    frame[i]=(temp);
  }
  frame[5]=(crc);
  // The command transmission always starts with the most significant bit(MSB).
  // The command format is: cmd, [32..0]data, crc
  for(i=0;i<6;i++) mmcSendByte(frame[i]);	// Send command.
}


/**********************************************************************************/
/*  Function name: mmcGetResponse			                                      */
/*  	Parameters                                                                */
/*          Input   :  No				                                          */
/*          Output  :  response					                          		  */
/*	Action: Wait response after commands or data are send to SD/MMC via SPI.	  */
/*	Author: Ti, descriptions and modifications by Penko Bozhkov.				  */
/*	Modifications:	1. Added 0xFE response, which is Start Block Token at  		  */
/*					   Single Block Read and Single Block Write Transactions. 	  */
/*					   Thus mmcGetXXResponse() function is rejected.			  */
/*					2. int i -> unsigned char i, 8-bits length instead of 16-bits */
/**********************************************************************************/
char mmcGetResponse(void)
{
  //Response comes 1-8bytes after command
  //the first bit will be a 0
  //followed by an error code
  //data will be 0xFF until response
  unsigned char i=0;
  char response;
	
  while(i<=64)
  {
   response = mmcSendByte(0xFF);
   if(response==0x00)break;
   if(response==0x01)break;
   if(response==0xFE)break;
   i++;
  }
  return response;
}


/**********************************************************************************/
/*  Function name: initMMC					                                      */
/*  	Parameters                                                                */
/*          Input   :  No	   				                                      */
/*          Output  :  Response: MMC_INIT_FAIL or MMC_INIT_SUCCESS         		  */
/*	Action: SD/MMC card initialization.							  				  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. int i -> unsigned char i, 8-bits length instead of 16-bits */
/*					2. Added int CMD1_counter, i.e. defined CMD1 time-out.		  */
/**********************************************************************************/
char initMMC(void)
{
  unsigned char i;
  int CMD1_counter = 0x00;
  char response = 0x01;

  // Start MMC initialization:
  //1. Send initialization sequence on PowerUp.
  //	The sequence length is the maximum of 1 ms, 74 clocks or the supply ramp-up time;
  //	the additional 10 clocks (over the 64 clocks after what the card should be ready for communication)
  //	are provided to eliminate power-up synchronization problems. [p.g. 20]
  //Raise NSS and MOSI for 74 clock cycles according to specification. Really for 90 clocks
  MMC_NSS_High;
  for(i=0;i<=9;i++)mmcSendByte(0xFF);
  //2. Put NSS low and Send Command 0 to put MMC in SPI mode
  // The CS signal must be continuously active for the duration of the SPI transaction (command, response and data). [p.g.66]
  MMC_NSS_Low;
  // The card will enter SPI mode if the CS signal is asserted (negative) during the reception of the reset command (CMD0).
  mmcSendCmd(0x00,0,0x95);	// Send COM0 software reset	[p.g.69]
  // If SPI mode is required (CS signal low), the module will switch to SPI mode and respond with the SPI mode R1 response. [p.g. 68/69]
  //3. Wait mmc to send R1 response, initialization process not completed 0x01;
  if(mmcGetResponse()!=0x01)	return MMC_INIT_FAIL;	// Response error
	// After power-on reset or software reset (CMD0), the card enters an idle state. In this state, the only legal
	// host commands are CMD1 (SEND_OP_COND) and CMD58 (READ_OCR).
	// The host must poll the card by repeatedly sending CMD1 until the in-idle-state bit in the card response indicates,
	// by being set to 0, that the card completed its initialization processes and is ready for the next command. [p.g.73]
  //4. Send repeatedly CMD1 until the in-idle-state bit is cleared
  while(response != 0x00)		// Response to CMD1 is R1 response!
  {	
   mmcSendCmd(0x01,0x00,0xFF);	// Send CMD1 continuously until in-idle-state bit is cleared i.e.
   response = mmcGetResponse();	// card completed its initialization processes and is ready for the next command.
   CMD1_counter++;
   if(CMD1_counter == 1000)	return MMC_INIT_FAIL;
  }
  //5. End of Initialization
  MMC_NSS_High;
  mmcSendByte(0xFF);			// Send 8 Clock pulses of delay.
  return MMC_INIT_SUCCESS;
}


/**********************************************************************************/
/*  Function name: mmcSetBlockLength		                                      */
/*  	Parameters                                                                */
/*          Input   :  blocklength (in bytes)		                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Send CMD16: SET_BLOCKLEN to SD/MMC card via SPI.	  				  */
/*						maximal block length is 512 bytes!!!					  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. long int blocklength -> unsigned int blocklength			  */
/*					2. Added: MMC_SetBL_ERROR if R1 response fail.			  	  */
/**********************************************************************************/
char mmcSetBlockLength(const unsigned int blocklength)
{
	// Max. read data block length and Max. write data block length according to CSD Register Fields
	// is 512 bytes, i.e. one sector (512-byte sector)
  	// Set the block length to read
  	mmcSendCmd(16, blocklength, 0xFF);	// Send CMD16
  	// get response from MMC - make sure that its 0x00 (R1 ok response format)
  	if(mmcGetResponse()!=0x00){
		// Send 8 Clock pulses of delay.
  		mmcSendByte(0xFF);
		return MMC_SetBL_ERROR;
	}
  	// Send 8 Clock pulses of delay.
  	mmcSendByte(0xFF);
  	return MMC_SetBL_SUCCESS;
}


/**********************************************************************************/
/*  Function name: mmcReadRegister			                                      */
/*  	Parameters                                                                */
/*          Input   :  cmd_register, length_in_bytes                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Read SD/MMC OCR,CID or CSD register value  via SPI.	  				  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. Function: mmcGetXXResponse() -> rejected.				  */
/*					2. Added "mmcReg_buffer" and corrected register read 	  	  */
/*					   direction, first read byte is register MSByte!!!			  */
/*					3. Added OCR register read function!						  */
/**********************************************************************************/
char mmcReadRegister(const char cmd_register, const unsigned char length_in_bytes)
{
  unsigned char uc = 0;
  char rvalue = MMC_RESPONSE_ERROR;
  // In SPI Mode only CID, CSD and OCR register are accessible!!! [p.g. 68]
  // Reading the contents of the OCR,CID or CSD registers in SPI mode is a simple read-block transaction.
  // The card will respond with a standard response token followed by a data block of 16 bytes,
  // suffixed with a 16-bit CRC. [p.g. 72]

  MMC_NSS_Low;	// Set CS low, i.e. select card
  if (mmcSetBlockLength (length_in_bytes) == MMC_SetBL_SUCCESS)
  {
  	// Send CMD register read
    // CRC not used: 0xFF as last byte
	// Read and write commands have data transfers associated with them. Data is being
	// transmitted or received via data tokens. All data bytes are transmitted MSB first. [p.g. 80]
    mmcSendCmd(cmd_register, 0x00000000, 0xFF);
    // wait for response
    // in the R1 format (0x00 is no errors)
    if (mmcGetResponse() == 0x00)
	{
		if(cmd_register != 58){	// If CID or CSD are read registers:
			// After R1 response, First read byte is: Start Block Token(At Single Block Read it is: 0xFE). [p.g. 80]
			if(mmcGetResponse() == 0xFE){
				// The next bytes are: 2-(N+1): User data. [p.g. 80]
	    		for (uc = 0; uc < length_in_bytes ; uc++){
					mmcReg_buffer[15-uc] = mmcSendByte(0xFF);
				}
				// And Last two bytes are: 16-bit CRC. [p.g. 80]
			  	// get CRC bytes (not really needed by us, but required by MMC)
		  		mmcSendByte(0xFF);
	  			mmcSendByte(0xFF);
				rvalue = MMC_ReadREG_SUCCESS;
			}
			else 	rvalue = MMC_RESPONSE_ERROR;
			// Error: Card hasn't sent Start Block Token.
		}
		else{	// If read register is OCR, i.e. CMD58
			// register read operation is: <39..31>R1, <31..0>OCR -> R3 format
			for (uc = 0; uc < length_in_bytes ; uc++){
				mmcReg_buffer[15-uc] = mmcSendByte(0xFF);
			}
			// And Last two bytes are: 16-bit CRC. [p.g. 80]
		  	// get CRC bytes (not really needed by us, but required by MMC)
		  	mmcSendByte(0xFF);
	  		mmcSendByte(0xFF);
			rvalue = MMC_ReadREG_SUCCESS;
		}
	}
    else rvalue = MMC_RESPONSE_ERROR;
	// Error: In card R1 response, or card hasn't sent R1 response.
    // Send 8 Clock pulses of delay.
    mmcSendByte(0xFF);
  }
  else rvalue = MMC_SetBL_ERROR;
  // Send 8 Clock pulses of delay.
  mmcSendByte(0xFF);
  MMC_NSS_High;
  return rvalue;
}				// mmc_read_register


/**********************************************************************************/
/*  Function name: mmcCapacity				                                      */
/*  	Parameters                                                                */
/*          Input   :  No				                                          */
/*          Output  :  mmcMemory_size or mmcMemory_ERROR                          */
/*	Action: Calculate MMC Capacity.							 			 		  */
/*	Author:	Penko Bozhkov														  */
/**********************************************************************************/
char mmcCapacity(void){
	unsigned int	C_SIZE;
	unsigned char	C_SIZE_MULT;
	unsigned char	READ_BL_LEN;
	unsigned int	MULT;
	unsigned int	BLOCK_LEN;
	unsigned long	BLOCKNR;
	unsigned long	Memory_capacity;
	unsigned char	Temp;
	
	MMC_NSS_Low;	// Set CS low, i.e. select card
	//extract READ_BL_LEN from mmcReg_buffer
	READ_BL_LEN = mmcReg_buffer[10] & 0x0F;
	if(READ_BL_LEN > 11)	return mmcMemory_Calculation_Error;
	//extract C_SIZE_MULT from mmcReg_buffer
	C_SIZE_MULT = mmcReg_buffer[6] &0x03;
	C_SIZE_MULT = C_SIZE_MULT << 1;
	if(mmcReg_buffer[5] & 0x80)	C_SIZE_MULT++;
	//extract C_SIZE from mmcReg_buffer
	C_SIZE = (mmcReg_buffer[9] & 0x03) << 8;
	C_SIZE = (C_SIZE | mmcReg_buffer[8]) << 2;
	Temp = (mmcReg_buffer[7] & 0xC0) >> 6;
	C_SIZE = C_SIZE | Temp;					// Length is 12 bits, so max value is: 0xFFF = 4095
	if(C_SIZE > 4095)	return mmcMemory_Calculation_Error;

	// Insert equations from SanDisk datasheet: ProdManRS-MMCv1.3 page 31
	Temp = C_SIZE_MULT + 2;
	MULT = 1 << Temp;	// This is the true equation! There is an error in datasheet! MULT Max value: 512
	BLOCK_LEN = 1 << READ_BL_LEN;								// Max value: 2^11 = 2048(0x800)
	BLOCKNR = (C_SIZE+1) * ((unsigned long)MULT);				// Max value: (4095+1)*512 = 2097152(0x200000)
	Memory_capacity = BLOCKNR * ((unsigned long)BLOCK_LEN);		// Max value: 2097152*2048 = 4294967296(0x100000000)
	MMC_NSS_High;	// Set CS high, i.e. deselect card

	if(Memory_capacity <= 63000000) 			return mmcMemory_less_than_64MB;
	else if(Memory_capacity <= 65000000)		return mmcMemory_64MB;
	else if(Memory_capacity <= 129000000)		return mmcMemory_128MB;
	else if(Memory_capacity <= 257000000)		return mmcMemory_256MB;
	else if(Memory_capacity <= 513000000)		return mmcMemory_512MB;
	else if(Memory_capacity <= 1100000000)		return mmcMemory_1GB;
	else if(Memory_capacity <= 2100000000)		return mmcMemory_2GB;
	else if(Memory_capacity <= 4100000000)		return mmcMemory_4GB;
	else 										return mmcMemory_more_than_4GB;
}


/**********************************************************************************/
/*  Function name: mmcCheckBusy				                                      */
/*  	Parameters                                                                */
/*          Input   :  No							                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Get SD/MMC Data Response Token, because every data block written to   */
/*				the card is acknowledged by a data response token. [p.g. 80],and  */
/*				if the data block is received with no errors wait card to stop	  */
/*				sending of busy tokens stream.									  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. int i -> unsigned int i									  */
/*					2. Added error response: card doesn't stop sending busy token.*/
/**********************************************************************************/
char mmcCheckBusy(void)
{
  unsigned int i=0;
  char response;
  char rvalue;
  // After a data block is received, the MultiMediaCard/RS-MultiMediaCard will respond with a data-response
  // token, and if the data block is received with no errors, it will be programmed. A continuous stream of
  // busy tokens will be sent to the host (effectively holding the dataOut line low) for the duration  of
  // time the card is busy, programming. [p.g. 71]

  // Get Card Data Response Token:
  while(i<=64)
  {
  	// Waiting Data Response from card
	// Data Response Format: xxx0<status>1
	// Status = 010 Data accepted.
	// Status = 101 Data rejected due to a CRC error.
	// Status = 110 Data rejected due to a write error.
   	response = mmcSendByte(0xFF);
   	response &= 0x1F;
   	switch(response)
    {
     	case 0x05: rvalue = MMC_WRITE_SUCCESS;break;
     	case 0x0B: return	MMC_CRC_ERROR;
     	case 0x0D: return	MMC_WRITE_ERROR;
     	default:
	      rvalue = MMC_OTHER_ERROR;
	      break;
    }
    if(rvalue==MMC_WRITE_SUCCESS)break;
    i++;
  }
  i=0;
  // Wait until card stop sending busy token stream:
  // because a continuous stream of busy tokens will be sent to the host,
  // (effectively holding the dataOut line low) for the duration of time the card is busy, programming.
  do
  {
   response = mmcSendByte(0xFF);
   i++;
   if(i == 65000){
   	// Error due to card doesn't stop sending of busy token stream.
   	rvalue = MMC_OTHER_ERROR;
	break;
   }
  }while(response==0);
  return rvalue;
}


/**********************************************************************************/
/*  Function name: mmcWriteBlock			                                      */
/*  	Parameters                                                                */
/*          Input   :  address						                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Write block of 512 bytes(one sector) in SD/MMC via SPI.	  			  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. unsigned long i -> unsigned int i 						  */
/**********************************************************************************/
char mmcWriteBlock(const unsigned long address)
{
  unsigned int i = 0;
  char rvalue = MMC_RESPONSE_ERROR;	// MMC_SUCCESS;
  //char c = 0x00;

  MMC_NSS_Low;	// Set CS low, i.e. select card
  // Set the block length to read
  if (mmcSetBlockLength(512) == MMC_SetBL_SUCCESS)	// block length could be set
  {
    // send write command
    mmcSendCmd (24,address, 0xFF);
    // check if the MMC acknowledged the write block command
    // it will do this by sending an affirmative response R1
    // in the R1 format (0x00 is no errors)
    if (mmcGetResponse() == 0x00)
	{
	  	// send the data token to signify the start of the data
	  	mmcSendByte(0xFE);	// Send Start Block Token
	  	// clock the actual data transfer and transmit the bytes
	  	for (i = 0; i < 512; i++)	mmcSendByte(mmc_write_buffer[i]);	// mmc_buffer[i];       Test: i & 0xff
	  	// put CRC bytes (not really needed by us, but required by MMC)
	  	mmcSendByte(0xFF);
	  	mmcSendByte(0xFF);
		// Wait 8-bits Data Response. Data Response Format is: xxx0<status>1 [p.g. 80]
	  	// Read the data response:
		// status 010: Data accepted;
		// status 101: Data rejected due to a crc error;
		// status 110: Data rejected due to a Write error.
		rvalue = mmcCheckBusy();
	}
    else{
	  	// the MMC never acknowledge the write command
	  	rvalue = MMC_RESPONSE_ERROR;	// 2
	}
  }
  else{
     rvalue = MMC_BLOCK_SET_ERROR;	// 1
  }
  MMC_NSS_High;	// Set CS high, i.e. deselect card
  return rvalue;
}				// mmc_write_block


/**********************************************************************************/
/*  Function name: mmcRead_VerifyBlock		                                      */
/*  	Parameters                                                                */
/*          Input   :  address, count				                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Read and verify block of 512 bytes(one sector) writed in SD/MMC.	  */
/*			This function reject using of below to functions and escape using of  */
/*			read buffer, which take very big place in data memory. If it is 	  */
/*			needed below two functions can be used!								  */
/*	Author: Penko Bozhkov.		  												  */
/**********************************************************************************/
char mmcRead_VerifyBlock(const unsigned long address, const unsigned int count)
{
  unsigned int i = 0;
  unsigned char temp;
  unsigned char compare;
  char rvalue = MMC_RESPONSE_ERROR;

  MMC_NSS_Low;	// Set CS low, i.e. select card
  // Set the block length to read
  if (mmcSetBlockLength(count) == MMC_SetBL_SUCCESS)	// block length could be set
  {
    // send read command MMC_READ_SINGLE_BLOCK=CMD17
    mmcSendCmd (17,address, 0xFF);
    // Check if the MMC acknowledged the read block command
    // it will do this by sending an affirmative response R1 to CMD17
    // in the R1 format (0x00 is no errors)
    if (mmcGetResponse() == 0x00)
	{
	  	// now look for the Single Block Read (0xFE) token to signify the start of the data
	  	if (mmcGetResponse() == MMC_START_DATA_BLOCK_TOKEN)
	    {
	    	// clock the actual data transfer and receive the bytes;
	      	for (i = 0; i < 512; i++){
				temp = mmcSendByte(0xFF);
				compare = temp^mmc_write_buffer[i];
				if(compare){
					rvalue = MMC_Read_Write_Verification_Fail;
					return rvalue;
				}	
			}	
	      	// get CRC bytes (not really needed by us, but required by MMC)
	      	mmcSendByte(0xFF);
	      	mmcSendByte(0xFF);
	      	rvalue = MMC_Read_Write_Verification_Success;
	    }
	  	else{
	      	// the data token was never received
	      	rvalue = MMC_READ_DATA_TOKEN_ERROR;	// 3
	    }
	}
    else{
	  	// the MMC never acknowledge the read command
	  	rvalue = MMC_READ_RESPONSE_ERROR;	// 2
	}
  }
  else{
  	// the MMC never acknowledge the Block Set Command
    rvalue = MMC_READ_BLOCK_SET_ERROR;	// 1
  }
  mmcSendByte(0xFF);
  MMC_NSS_High;	// Set CS high, i.e. deselect card
  return rvalue;
}				// mmc_read_block


/**********************************************************************************/
/*  Function name: mmcReadBlock				                                      */
/*  	Parameters                                                                */
/*          Input   :  address, count				                              */
/*          Output  :  rvalue					                          		  */
/*	Action: Read block of 512 bytes(one sector) written in SD/MMC via SPI.		  */
/*	Author: Rolf Freitag, descriptions and modifications by Penko Bozhkov.		  */
/*	Modifications:	1. unsigned long count -> unsigned int count 				  */
/**********************************************************************************/
/*
char mmcReadBlock(const unsigned long address, const unsigned int count)
{
  unsigned int i = 0;
  char rvalue = MMC_RESPONSE_ERROR;

  MMC_NSS_Low;	// Set CS low, i.e. select card
  // Set the block length to read
  if (mmcSetBlockLength(count) == MMC_SetBL_SUCCESS)	// block length could be set
  {
    // send read command MMC_READ_SINGLE_BLOCK=CMD17
    mmcSendCmd (17,address, 0xFF);
    // Check if the MMC acknowledged the read block command
    // it will do this by sending an affirmative response R1 to CMD17
    // in the R1 format (0x00 is no errors)
    if (mmcGetResponse() == 0x00)
	{
	  	// now look for the Single Block Read (0xFE) token to signify the start of the data
	  	if (mmcGetResponse() == MMC_START_DATA_BLOCK_TOKEN)
	    {
	    	// clock the actual data transfer and receive the bytes;
	      	for (i = 0; i < 512; i++){
				mmc_read_buffer[i]= mmcSendByte(0xFF);
			}	
	      	// get CRC bytes (not really needed by us, but required by MMC)
	      	mmcSendByte(0xFF);
	      	mmcSendByte(0xFF);
	      	rvalue = MMC_READ_SUCCESS;
	    }
	  	else{
	      	// the data token was never received
	      	rvalue = MMC_READ_DATA_TOKEN_ERROR;	// 3
	    }
	}
    else{
	  	// the MMC never acknowledge the read command
	  	rvalue = MMC_READ_RESPONSE_ERROR;	// 2
	}
  }
  else{
  	// the MMC never acknowledge the Block Set Command
    rvalue = MMC_READ_BLOCK_SET_ERROR;	// 1
  }
  MMC_NSS_High;	// Set CS high, i.e. deselect card
  return rvalue;
}				// mmc_read_block
*/

/**********************************************************************************/
/*  Function name: mmcRead_WriteVerification                                      */
/*  	Parameters                                                                */
/*          Input   :  No				                                          */
/*          Output  :  MMC_Read_Write_Verification_Fail(Success)                  */
/*	Action: Verify written block of 512 Bytes .				 			 		  */
/*	Author:	Penko Bozhkov														  */
/**********************************************************************************/
/*
char mmcRead_WriteVerification(void){
	unsigned char compare;
	
	MMC_NSS_Low;	// Set CS low, i.e. select card
	for(int i=0; i<512; i++){
		compare = mmc_read_buffer[i]^mmc_write_buffer[i];
		if(compare)	return MMC_Read_Write_Verification_Fail;
	}
	MMC_NSS_High;	// Set CS high, i.e. deselect card
	return MMC_Read_Write_Verification_Success;
}
*/





